package gs.smolban.db

import cats.data.EitherT
import gs.smolban.model.CreatedAt
import gs.smolban.model.Tag

trait TagDb[F[_]]:

  /** Create a new [[Tag]].
    *
    * @param tag
    *   The value of the tag to create (some arbitrary string).
    * @param createdAt
    *   The instant at which this tag was created.
    * @return
    *   The new [[Tag]], or an error if the value already exists.
    */
  def createTag(
    tag: Tag.Value,
    createdAt: CreatedAt
  ): EitherT[F, DbError, Tag]

  /** Get the value of some [[Tag]] by ID.
    *
    * @param id
    *   The internal identifier of the [[Tag]].
    * @return
    *   The found [[Tag]], or `None` if the ID does not exist.
    */
  def readTag(id: Tag.Id): F[Option[Tag]]

  /** Delete a [[Tag]] by ID.
    *
    * @param id
    *   The unique ID of the [[Tag]] to delete.
    * @return
    *   `true` if the [[Tag]] was deleted, `false` otherwise.
    */
  def deleteTag(id: Tag.Id): F[Boolean]
